from flask import json, Response
from typing import Dict


class ApiResponse(Response):

    def __init__(self, data: Dict, status: int = 200, *,
                 warnings=None, meta=None, links=None, headers=None):

        resp = {
            'data': data,
            'warnings': warnings,
            '_meta': meta,
            '_links': links
        }

        super().__init__(json.dumps({k: v for (k, v) in resp.items()
                                     if v is not None and len(v) > 0}),
                         status=status, mimetype='application/json', headers=headers)



import sys
from flask import Response, json, current_app
from werkzeug.exceptions import HTTPException
from typing import Dict, List


class ApiError(HTTPException):
    def __init__(self, message: str, status: int = 500, *, extra: Dict = None, headers: List = None):
        """ If used with flask-restful, the data field is being returned"""
        self._headers = headers

        resp = {
            'message': message,
            'extra': extra
        }

        self.code = status
        self.description = resp['message']  # to make it compliant with general HTTPException use
        self.data = {k: v for k, v in resp.items() if v is not None}

    def to_dict(self):
        return self.data

    def get_response(self, environ=None):
        return Response(json.dumps(self.to_dict()),
                        status=self.code, mimetype='application/json',
                        headers=self._headers)

    def handle_error(self):
        """log and return response"""
        if self.code and self.code >= 500:
            exc_info = sys.exc_info()
            if exc_info[1] is None:
                exc_info = None
            current_app.log_exception(exc_info)
        return self.get_response()


def register_api_error(app):
    app.register_error_handler(ApiError, lambda err: err.handle_error())



import wrapt
from inspect import getfullargspec
from flask import request
from marshmallow import EXCLUDE
from marshmallow.exceptions import ValidationError
from werkzeug.exceptions import BadRequest

from cruet import ApiError


def dataschema(schema, format: str = 'json', unknown=EXCLUDE, error_class=ApiError):
    """ Decorator for input validation of endpoints.
    Requires marshmallow 3
    """
    @wrapt.decorator
    def wrapper(wrapped, instance, args, kwargs):
        if format == 'json':
            try:
                data = request.get_json()
                if data is None:
                    if request.mimetype != 'application/json':
                        raise error_class(
                            'Mimetype application/json expected but {} found'.
                            format(request.mimetype), 400)
                    raise error_class('No JSON found in the request body', 400)
            except BadRequest as e:
                raise error_class('The payload must be valid json and ' +
                               'the mimetype "application/json"', 400,
                               extra=e.description)
        elif format == 'param':
            data = request.values.to_dict()
        else:
            raise RuntimeError('format has to be either param or json')

        try:
            data_validated = schema.load(data, unknown=unknown)
            keys_excluded = (data.keys() - data_validated.keys())\
                if unknown == EXCLUDE else set()

        except ValidationError as e:
            raise error_class('Validation of input failed', 400, extra=e.messages)

        # update keyword args for the decorated function
        new_kwargs = {}
        wrapped_args = getfullargspec(wrapped).args
        if 'data' in wrapped_args:
            new_kwargs['data'] = data_validated

        if 'excluded' in wrapped_args:
            new_kwargs['excluded'] = keys_excluded

        kwargs.update(new_kwargs)

        return wrapped(*args, **kwargs)
    return wrapper



"""
cruet
---------
cruet is a toolkit for creating microservices with flask.
"""
import io
import re
from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

with io.open("cruet/__init__.py", "rt", encoding="utf8") as f:
    version = re.search(r"__version__ = \"(.*?)\"", f.read()).group(1)

setup(
    name="cruet",
    version=version,
    license="BSD",
    author="Scientific Computing - Australian Synchrotron",
    author_email="ascomputing@ansto.gov.au",
    description="A toolkit for building microservices with flask",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/AustralianSynchrotron/cruet",
    packages=find_packages(),
    install_requires=[
        "Flask>=1.0.3",
        "wrapt>=1.11.1",
        "marshmallow>=3.0.0rc6"
    ],
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Environment :: Web Environment",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
)
